<?php

class ZippyCourses_CorePages_Actions
{
    protected static $instance;
    public $logout_redirect_url;
    public $dashboard_redirect_url;

    public static function instance()
    {
        if (self::$instance === null) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    private function __construct()
    {
        $zippy = Zippy::instance();
        $this->logout_redirect_url = apply_filters('zippy_courses_logout_redirect_url', home_url());


        add_action('template_redirect', array($this, 'login'), 1);
        add_action('template_redirect', array($this, 'logout'), 1);
        add_action('template_redirect', array($this, 'register'));
        add_action('template_redirect', array($this, 'studentOnly'), 9);
    }

    public function logout()
    {
        global $post;

        if (!is_object($post)) {
            return;
        }

        $zippy = Zippy::instance();

        if ($zippy->core_pages->is($post->ID, 'logout')) {
            $zippy->sessions->flashMessage(__('You have been logged out.', ZippyCourses::TEXTDOMAIN));

            wp_logout();
            wp_redirect($this->logout_redirect_url);
            exit;
        }
    }

    public function login()
    {
        global $post;

        if (!is_object($post)) {
            return;
        }

        $zippy = Zippy::instance();

        if ($zippy->core_pages->is($post->ID, 'login') && is_user_logged_in()) {
            wp_redirect($zippy->core_pages->getUrl('dashboard'));
            exit;
        }
    }

    public function register()
    {
        global $post, $current_user;

        if (!is_object($post)) {
            return;
        }

        $zippy = Zippy::instance();
        
        if ($zippy->core_pages->is($post->ID, 'register') && is_user_logged_in()) {
            $student = $zippy->make('student', array($current_user->ID));
            $claim = filter_input(INPUT_GET, 'claim', FILTER_SANITIZE_STRING);

            if (!empty($claim)) {
                $product = $zippy->utilities->product->getByUid($claim);

                if ($product === null) {
                    $transaction = $zippy->make('transaction');
                    $transaction->buildByTransactionKey($claim);

                    $zippy->access->claim($student, $transaction);
                } else {
                    $zippy->access->grant($current_user->ID, $product->getId());
                    $courses = $product->getCourses();
                    foreach ($courses as &$course) {
                        $course = get_the_title($course);
                    }
                    
                    $zippy->sessions->flashMessage(
                        sprintf(
                            __('You have joined the following course(s): %s', ZippyCourses::TEXTDOMAIN),
                            '<strong>' . implode(', ', $courses) . '</strong>'
                        )
                    );
                }
            }

            wp_redirect($zippy->core_pages->getUrl('dashboard'));
            exit;
        }
    }

    public function studentOnly()
    {
        global $post;

        if (!is_object($post) || is_front_page() || is_home()) {
            return;
        }

        $zippy = Zippy::instance();

        $student = $zippy->cache->get('student');
        $needs_redirect = $student === null
            ? array_values($zippy->core_pages->getPrivateOnlyCorePageIds())
            : array_values($zippy->core_pages->getPublicOnlyCorePageIds());

        if (in_array($post->ID, $needs_redirect)) {
            if ($student === null) {
                $zippy->sessions->flashMessage(
                    __('You must be logged in to view that page.', ZippyCourses::TEXTDOMAIN),
                    'error'
                );
                wp_redirect($zippy->core_pages->getUrl('login') . '?redirect_to=' . urlencode(get_permalink($post->ID)));
                exit;
            } else {
                $zippy->sessions->flashMessage(
                    __('You are already logged in!', ZippyCourses::TEXTDOMAIN),
                    'warning'
                );
                wp_redirect($zippy->core_pages->getUrl('dashboard'));
                exit;
            }

        }
    }
}
