<?php

class ZippyCourses_SecureDownload
{
    public function __construct()
    {
        add_action('template_redirect', array($this, 'download'), 1);
    }

    public function download()
    {
        $zippy = Zippy::instance();

        $student = $zippy->cache->get('student');

        $uid = get_query_var('zippy_file_uid', false);
        if (!$uid) {
            return;
        }

        $download = $this->_getDownload($uid);

        if ($download === null) {
            //
        }

        $url        = $download['url'];
        $entry_id   = $download['entry_id'];

        $entry = new stdClass;
        $entry->id = $entry_id;

        $middleware = $zippy->middleware->get('student-access');

        // Apply filter so that we can override the student access middleware if necessary
        $has_student_access = apply_filters(
            'zippy_downloads_have_student_access',
            $middleware->run($entry),
            $student,
            $entry
        );
        if ($uid && $has_student_access) {
            $zippy->activity->insert($student->getId(), $uid, 'download');

            $fs  = $this->_getFileStream($url);
            $fs->download();
        } else {
            $zippy->sessions->flashMessage(
                __('You do not have permission to download this file.', ZippyCourses::TEXTDOMAIN),
                'error'
            );

            wp_redirect(home_url());
            exit;
        }
    }

    public function _getUrl($uid)
    {
        $zippy = Zippy::instance();

        $downloads = $zippy->utilities->entry->getAllDownloads();

        if (isset($downloads[$uid]) && isset($downloads[$uid]['url'])) {
            return $downloads[$uid]['url'];
        }

        return false;
    }

    public function _getDownload($uid)
    {
        $zippy = Zippy::instance();

        $downloads = $zippy->utilities->entry->getAllDownloads();

        return isset($downloads[$uid]) ? $downloads[$uid] : null;
    }

    public function _getFileStream($url)
    {
        $method = function_exists('curl_exec') ? 'curl' : 'socket';

        return apply_filters('zippy_filter_download_method', $method) == 'curl' ? new Zippy_Curl_FileStream($url) :  new Zippy_Socket_FileStream($url);
    }
}
