<?php

class Zippy_StudentHasProgressAccess_MiddlewareRule extends Zippy_MiddlewareRule
{
    public function __construct()
    {
        $this->setErrorMessage(__('You must complete a previous lesson.', ZippyCourses::TEXTDOMAIN));
        $this->id = 'student-has-progress-access';
    }

    public function handle($object)
    {
        $zippy = Zippy::instance();

        $student    = $zippy->cache->get('student');
        $id         = isset($object->ID) ? $object->ID : $object->id;
        
        if ($this->checkCache($object) !== null) {
            return $this->handleCachedResult($object, $this->checkCache($object));
        }
        
        if ($student == null || get_post_type($id) == 'course') {
            return $this->pass($object);
        }

        $course_id = $zippy->utilities->entry->getCourseId($id);
        $entry_ids = $zippy->utilities->course->getCourseEntryIds($course_id);

        if (!in_array($id, $entry_ids)) {
            return $this->pass($object);
        }

        $course = $student->getCourse($course_id);

        if ($course === null) {
            return $this->pass($object);
        }

        $access = true;
        $requires_previous_lesson_completed = false;

        $entry_ids = $course->getEntryIds();
        $key = array_search($id, $entry_ids);

        $exempted_test = $course->entries->getRecursive($id);
        if ($exempted_test !== null) {
            if ($exempted_test->isExempt()) {
                return $this->pass($object);
            }
        }

        $segment = array_splice($entry_ids, 0, $key);

        $required = array();

        foreach ($segment as $entry_id) {
            $entry = $course->entries->getRecursive($entry_id);

            if ($entry->isRequired()) {
                $required[] = $entry_id;
            }
        }

        foreach ($segment as $entry_id) {
            if ($entry_id == $id) {
                break;
            }

            if (!in_array($entry_id, $required)) {
                continue;
            } else {
                $access = $student->isCompleted($entry_id);
            }

            if (!$access) {
                break;
            }
        }
        
        return $access ? $this->pass($object) : $this->fail($object);
    }
}
